/* tslint:disable */
/* eslint-disable */
export function parse_powershell(code: string): string;
export function parse_assets_sql(code: string): string;
export function parse_snowflake(code: string): string;
export function parse_db_resource(code: string): string | undefined;
export function parse_graphql(code: string): string;
export function parse_mssql(code: string): string;
export function parse_sql(code: string): string;
export function parse_oracledb(code: string): string;
export function parse_bigquery(code: string): string;
export function parse_mysql(code: string): string;
export function parse_bash(code: string): string;
export function parse_duckdb(code: string): string;
